function Satel() {

}
    
//init
Satel.prototype.init = function(){
    this.device = IR.GetDevice("Custom Driver");
    this.mainPage = IR.GetPage("mainPage");
    this.mainList = this.mainPage.GetItem("list");
    
    this.onlineStatus = {   icon:    this.mainPage.GetItem("statusIcon").GetState(0),
                            label:   this.mainPage.GetItem("statusLabel").GetState(0)  };
    
    //list of commands to send   
    this.bSensors = [];
    
    //list of temperature sensors to send
    this.termoSensors = [];
    
    //recieved data from integra
    this.recievedBSensors = {};
    this.recievedTSensors = {};
    
//sending commands

    arrBSensors = [];
    arrTSensors = [];    
    var count = 0;
    //bSensors and TSensors summary length
    var totalLength = 0;
    
    //main timer fo refresh integra state
    IR.SetInterval(300, function(){
        //refresh integra connect status     
        this.updateOnline();
        
        //refresh data on each cicle
        if (count == 0){
            arrBSensors = this.bSensors;
            arrTSensors = this.termoSensors;
            totalLength = arrBSensors.length + arrTSensors.length;
        }
        
        //sending binary sensors first, than temperature sensors
        if (count < arrBSensors.length){  
            if (arrBSensors.length)
                this.device.Send(objBSensorsData[arrBSensors[count]]);
        } else {
            if (arrTSensors.length){
                var l_id = arrTSensors[count - arrBSensors.length];
                var l_command = createTempCommand(l_id);
                sendFrame(l_command);
            }
        }
        count ++;
        if (count >= totalLength)
            count = 0;
    }, this);

    //recieving data from Integra
    IR.AddListener(IR.EVENT_RECEIVE_DATA, this.device, function(_data){
        var command = _data[2];
        var data = _data.slice(3, -4);
        //binary sensors
        if (this.bSensors.indexOf(command) != -1){
            this.recievedBSensors[command] = data;
        }
        //temperature sensors
        if (command == 0x7D){
            var id = data.shift();
            this.recievedTSensors[id] = data;
        }
    }, this);
}

//================================================================================
//check Satel device connect status
Satel.prototype.checkDeviceStatus = function(){
    if (IR.GetVariable("Drivers.Custom Driver.Status") == 2)
        return true;
}

//================================================================================
//refresh connectivity icon and title
Satel.prototype.updateOnline = function(){
    if (this.checkDeviceStatus()){
        this.onlineStatus.icon.TextColor = 0x39B54AFF;
        this.onlineStatus.label.Text = "Online";
    } else {                                  
        this.onlineStatus.icon.TextColor = 0xFF0000FF;
        this.onlineStatus.label.Text = "Offline";
    }
}

//================================================================================
//new binary sensor add
Satel.prototype.addBSensor = function(name, id, type){
   new BSensor(name, id, type, this);        
}

//================================================================================   
//new button add
Satel.prototype.addButton = function(name, id, type){
    new Button(name, id, type, this.mainList);
}     

//================================================================================
//new security widget add
Satel.prototype.addSecurity = function(name, id, mode, btnNames){
    new Security(name, id, mode, btnNames, this.mainList);
} 

//================================================================================
//new relay add
Satel.prototype.addRelay = function(name, id){
    new Relay(name, id, this);
}

//================================================================================
//new temperature sensor add
Satel.prototype.addTSensor = function(name, id){
    new Temperature(name, id, this);
}

//==================================================================================
//                                FUNCTIONS
//==================================================================================                                                                      


//==================================================================================
//show error popup, _sLastPopup - popup name, where error is occur
function showError(_text, _sLastPopup){
    var errorPopup = IR.GetPopup("error");
    errorPopup.GetItem("label").Text = _text;
    
    IR.HideAllPopups();    
    IR.ShowPopup("error");

    
    IR.AddListener(IR.EVENT_ITEM_PRESS, IR.GetPopup("error").GetItem("ok"), function(){
        IR.HideAllPopups();
        IR.ShowPopup(_sLastPopup);   
    });  
}

//================================================================================
//popup fast switch: w1 - popup to hide, w2 - popup to show
function HideShow(w1,w2){  
    IR.HidePopup(w1);
    IR.ShowPopup(w2);
}

//================================================================================
//check the bit value in data, count from 1, not 0
//e.g. chteckState([0,0,2,0], 18) == true; ([2,0,0,0], 2]) == true;
//checkState([64,0,0,0], 7) == true; ect
function checkState(data, id){
    var pos = id % 8 - 1;
    var oktet = parseInt (id / 8);
    if (pos == -1){
        pos = 7;                              
        oktet--;
    }
    if (oktet > data.length - 1)
        IR.Log('checkState --- Id is larger than data length');  
    var sensorState = (data[oktet] >> pos) & 1;
    return sensorState;
}

//=================================================================================
//send frame to Satel
function sendFrame(data){
    var frame = new Array();
    
    frame.push(0xfe);
    frame.push(0xfe);
    
    for(var i=0; i<data.length; i++)
    {
        frame.push(data[i]);
        if(data[i]==0xfe) 
            frame.push(0xf0);
    }   
            
    var crc = crc16(data);
        
    frame.push((crc>>8) & 0xff);
        
    if(((crc>>8) & 0xff)==0xfe) 
        frame.push( 0xf0 );
            
    frame.push(crc & 0xff);
        
    if((crc & 0xff)==0xfe) 
        frame.push( 0xf0 );
            
    frame.push(0xfe);
    frame.push(0x0d);
    IR.GetDevice("Custom Driver").Send(frame);
}

//==========================================================================================
//generate crc16 checksumm 
function crc16(param){
    var crc=0x147a;
    for(w = 0; w < param.length; w++)
    {
        crc=((crc<<1) | (crc>>15)) & 0xFFFF;
        crc=crc ^ 0xffff & 0xffff;
        crc=crc+(crc>>8 & 0xff)+param[w];
        if( param[w]==0xfe && param[w+1]==0xf0 ) w++;
    }
    
    return(crc);
}

//==========================================================================================
//generate array of zeroes of given length, with a single bit set to 1
function setBit(id, len){
    
    var data = [];
    var oktet = parseInt (id / 8);               //which BYTE is needed
    var pos = id % 8 - 1;                        //which BIT is needed

    if (pos == -1){                              
        pos = 7;                                 
        oktet--;
    }                   
    for (var i = 0; i < oktet; i++){             //adding zeroes at the start
        data.push(0x00);
    }
    
    data.push(0x01 << pos);                      //set bit to 1
    for (var i = oktet + 1; i < len; i++){       //adding zeroes at the end
        data.push(0x00);
    }
    return data;   
}

//==========================================================================================
//filling given list with items

//in_oItems = [{name: 'aaa', value: 'bbb', visible: true},
//             {name: 'bbb', value: 'ccc', visible: false}]
//visible - show/not show VALUE
//if not need in_listBG, give 0

function fillList(in_list, in_sTemplate, in_oItems, in_listBG){
    var height = IR.GetPopup('selectTemplate').Height;
    
    for (var i = 0; i < in_oItems.length; i++){
        in_list.AddItem(in_sTemplate, { name: {Text: in_oItems[i].name}, 
                                        value: {Visible: in_oItems[i].visible, Text: in_oItems[i].value}});
    }
    
    if (in_listBG){
        if (in_list.ItemsCount > 7){
            in_list.Height = 7.5 * height;
            in_listBG.Height = in_list.Height + 10;
            in_list.ScrollBar = true;
            in_list.ScrollEnabled = true;
        } else {
            in_list.Height = in_oItems.length * height;
            in_listBG.Height = in_list.Height + 10;
        }
    }
}

//==========================================================================================
//return list items to default values
function resetList(in_list, in_oItems){
    for (var i = 0; i < in_list.ItemsCount; i++){
        in_list.GetItemByIndex(i).Popup.GetItem('value').Text = in_oItems[i].value;
        in_list.GetItemByIndex(i).Popup.GetItem('fog').Visible = false;        
    }
}

//==========================================================================================
//array search
Array.prototype.indexOf = function (searchElement, fromIndex) {
   var k;                                                                                                           
   if (this == null) {
      throw new TypeError('"this" is null or not defined');
   }
   var O = Object(this);
   var len = O.length >>> 0;
   if (len === 0) {
      return -1;
   }
   var n = +fromIndex || 0;
   if (Math.abs(n) === Infinity) {
      n = 0;
   }
   if (n >= len) {
      return -1;
   }
   k = Math.max(n >= 0 ? n : len - Math.abs(n), 0);                      
   while (k < len) {  
      if (k in O && O[k] === searchElement) {
         return k;
      }
      k++;
   }
   return -1;
};